<?php
/**
 * iCalcreator, a PHP rfc2445/rfc5545 solution.
 *
 * This file is a part of iCalcreator.
 *
 * Copyright (c) 2007-2017 Kjell-Inge Gustafsson, kigkonsult, All rights reserved
 * Link      http://kigkonsult.se/iCalcreator/index.php
 * Package   iCalcreator
 * Version   2.24
 * License   Subject matter of licence is the software iCalcreator.
 *           The above copyright, link, package and version notices,
 *           this licence notice and the [rfc5545] PRODID as implemented and
 *           invoked in iCalcreator shall be included in all copies or
 *           substantial portions of the iCalcreator.
 *           iCalcreator can be used either under the terms of
 *           a proprietary license, available at <https://kigkonsult.se/>
 *           or the GNU Affero General Public License, version 3:
 *           iCalcreator is free software: you can redistribute it and/or
 *           modify it under the terms of the GNU Affero General Public License
 *           as published by the Free Software Foundation, either version 3 of
 *           the License, or (at your option) any later version.
 *           iCalcreator is distributed in the hope that it will be useful,
 *           but WITHOUT ANY WARRANTY; without even the implied warranty of
 *           MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *           GNU Affero General Public License for more details.
 *           You should have received a copy of the GNU Affero General Public
 *           License along with this program.
 *           If not, see <http://www.gnu.org/licenses/>.
 */
namespace kigkonsult\iCalcreator\traits;
use kigkonsult\iCalcreator\util\util;
use kigkonsult\iCalcreator\util\utilGeo;
/**
 * GEO property functions
 *
 * @author Kjell-Inge Gustafsson, kigkonsult <ical@kigkonsult.se>
 * @since 2.22.23 - 2017-02-02
 */
trait GEOtrait {
/**
 * @var array component property GEO value
 * @access protected
 */
  protected $geo = null;
/**
 * Return formatted output for calendar component property geo
 *
 * @return string
 */
  public function createGeo() {
    if( empty( $this->geo ))
      return null;
    if( empty( $this->geo[util::$LCvalue] ))
      return ( $this->getConfig( util::$ALLOWEMPTY )) ? util::createElement( util::$GEO ) : null;
    return util::createElement( util::$GEO,
                                util::createParams( $this->geo[util::$LCparams] ),
                                utilGeo::geo2str2( $this->geo[util::$LCvalue][utilGeo::$LATITUDE],  utilGeo::$geoLatFmt ) .
                                util::$SEMIC .
                                utilGeo::geo2str2( $this->geo[util::$LCvalue][utilGeo::$LONGITUDE], utilGeo::$geoLongFmt ));
  }
/**
 * Set calendar component property geo
 *
 * @param mixed $latitude
 * @param mixed $longitude
 * @param array $params
 * @return bool
 */
  public function setGeo( $latitude, $longitude, $params=null ) {
    if( isset( $latitude ) && isset( $longitude )) {
      if( ! is_array( $this->geo ))
        $this->geo = [];
      $this->geo[util::$LCvalue][utilGeo::$LATITUDE]  = floatval( $latitude );
      $this->geo[util::$LCvalue][utilGeo::$LONGITUDE] = floatval( $longitude );
      $this->geo[util::$LCparams] = util::setParams( $params );
    }
    elseif( $this->getConfig( util::$ALLOWEMPTY ))
      $this->geo = [util::$LCvalue  => util::$EMPTYPROPERTY,
                    util::$LCparams => util::setParams( $params )];
    else
      return false;
    return true;
  }
}
